@@ -13,8 +13,6 @@
 #![doc(html_logo_url = "https://docs.sequoia-pgp.org/logo.svg")]
 #![warn(missing_docs)]
 
-use base64;
-
 use std::convert::TryFrom;
 use std::io;
 use std::io::prelude::*;
@@ -98,7 +96,7 @@ pub struct AutocryptHeader {
 impl AutocryptHeader {
     fn empty(header_type: AutocryptHeaderType) -> Self {
         AutocryptHeader {
-            header_type: header_type,
+            header_type,
             key: None,
             attributes: Vec::new(),
         }
@@ -137,7 +135,7 @@ impl AutocryptHeader {
         for uidb in cert.userids().with_policy(policy, None) {
             // XXX: Fix match once we have the rfc2822-name-addr.
             if let Ok(Some(a)) = uidb.userid().email() {
-                if &a == addr {
+                if a == addr {
                     acc.push(uidb.userid().clone().into());
                     acc.push(uidb.binding_signature().clone().into());
                     found_one = true;
@@ -239,7 +237,7 @@ impl AutocryptHeaders {
             // Return any error.
             let mut line = line?;
 
-            if line == "" {
+            if line.is_empty() {
                 // End of headers.
                 break;
             }
@@ -252,8 +250,9 @@ impl AutocryptHeaders {
             //
             // See https://tools.ietf.org/html/rfc5322#section-2.2.3
             while let Some(Ok(nl)) = next_line {
-                if nl.len() > 0 && (&nl[0..1] == " " || &nl[0..1] == "\t") {
-                    line.push_str(&nl[..]);
+                if !nl.is_empty() && (nl.starts_with(|c| c == ' ' || c == '\t'))
+                {
+                    line.push_str(&nl);
                     next_line = lines.next();
                 } else {
                     // Put it back.
@@ -266,15 +265,14 @@ impl AutocryptHeaders {
             const AUTOCRYPT_GOSSIP : &str = "Autocrypt-Gossip: ";
             const FROM : &str = "From: ";
 
-            if line.starts_with(FROM) {
-                headers.from
-                    = Some(line[FROM.len()..].trim_matches(' ').into());
+            if let Some(rest) = line.strip_prefix(FROM) {
+                headers.from = Some(rest.trim_matches(' ').into());
             } else if line.starts_with(AUTOCRYPT) || line.starts_with(AUTOCRYPT_GOSSIP) {
                 headers.headers.push(Self::decode_autocrypt_like_header(&line));
             }
         }
 
-        return Ok(headers)
+        Ok(headers)
     }
 
     /// Decode header that has the same format as the Autocrypt header.
@@ -315,7 +313,7 @@ impl AutocryptHeaders {
                 }
             }
 
-            let critical = key.len() >= 1 && &key[0..1] == "_";
+            let critical = !key.is_empty() && &key[0..1] == "_";
             header.attributes.push(Attribute {
                 critical,
                 key: if critical {
@@ -468,11 +466,11 @@ impl AutocryptSetupMessage {
         let mut p : Vec<u8> = Vec::new();
         for i in 0..36 {
             if i > 0 && i % 4 == 0 {
-                p.push('-' as u8);
+                p.push(b'-');
             }
 
-            p.push(('0' as u8) + ((p_as_u128 as u8) % 10));
-            p_as_u128 = p_as_u128 / 10;
+            p.push(b'0' + ((p_as_u128 as u8) % 10));
+            p_as_u128 /= 10;
         }
 
         p.into()
@@ -508,11 +506,11 @@ impl AutocryptSetupMessage {
         let mut headers : Vec<(&str, &str)> = Vec::new();
         if let Some(ref format) = self.passcode_format {
             headers.push(
-                (&"Passphrase-Format"[..], &format[..]));
+                ("Passphrase-Format", &format[..]));
         }
         if let Some(ref begin) = self.passcode_begin {
             headers.push(
-                (&"Passphrase-Begin"[..], &begin[..]));
+                ("Passphrase-Begin", &begin[..]));
         }
 
         let mut armor_writer =
@@ -530,7 +528,7 @@ impl AutocryptSetupMessage {
             let mut w = armor::Writer::with_headers(
                 m, armor::Kind::SecretKey,
                 vec![("Autocrypt-Prefer-Encrypt",
-                      self.prefer_encrypt().unwrap_or(&"nopreference"[..]))])?;
+                      self.prefer_encrypt().unwrap_or("nopreference"))])?;
 
             self.cert.as_tsk().serialize(&mut w)?;
             let m = w.finalize()?;
@@ -544,8 +542,8 @@ impl AutocryptSetupMessage {
     /// Parses the autocrypt setup message in `r`.
     ///
     /// `passcode` is the passcode used to protect the message.
-    pub fn from_bytes<'a>(bytes: &'a [u8])
-        -> Result<AutocryptSetupMessageParser<'a>>
+    pub fn from_bytes(bytes: &[u8])
+        -> Result<AutocryptSetupMessageParser>
     {
         Self::from_reader(bytes)
     }
@@ -571,7 +569,7 @@ impl AutocryptSetupMessage {
                     if k == "Passphrase-Format" { Some(v) } else { None }
                 })
                 .collect::<Vec<&String>>();
-            let format = if format.len() > 0 {
+            let format = if !format.is_empty() {
                 // If there are multiple headers, then just silently take
                 // the first one.
                 Some(format[0].clone())
@@ -584,7 +582,7 @@ impl AutocryptSetupMessage {
                     if k == "Passphrase-Begin" { Some(v) } else { None }
                 })
                 .collect::<Vec<&String>>();
-            let begin = if begin.len() > 0 {
+            let begin = if !begin.is_empty() {
                 // If there are multiple headers, then just silently take
                 // the first one.
                 Some(begin[0].clone())
@@ -619,8 +617,7 @@ impl AutocryptSetupMessage {
             Packet::SKESK(skesk) => skesk,
             p => return Err(
                 Error::MalformedMessage(
-                    format!("Expected a SKESK packet, found a {}", p.tag())
-                        .into())
+                    format!("Expected a SKESK packet, found a {}", p.tag()))
                 .into()),
         };
 
@@ -636,8 +633,7 @@ impl AutocryptSetupMessage {
                     ref p => return Err(
                         Error::MalformedMessage(
                             format!("Expected a SEIP packet, found a {}",
-                                    p.tag())
-                                .into())
+                                    p.tag()))
                         .into()),
                 }
 
@@ -648,8 +644,8 @@ impl AutocryptSetupMessage {
         Ok(AutocryptSetupMessageParser {
             passcode_format: format,
             passcode_begin: begin,
-            skesk: skesk,
-            pp: pp,
+            skesk,
+            pp,
             passcode: None,
         })
     }
@@ -733,7 +729,7 @@ impl<'a> AutocryptSetupMessageParser<'a> {
                 p => return Err(Error::MalformedMessage(
                     format!("SEIP container contains a {}, \
                              expected a Literal Data packet",
-                            p.tag()).into()).into()),
+                            p.tag())).into()),
             }
 
             // The inner message consists of an ASCII-armored encoded
@@ -756,7 +752,7 @@ impl<'a> AutocryptSetupMessageParser<'a> {
                         })
                         .collect::<Vec<&String>>();
 
-                    if prefer_encrypt.len() > 0 {
+                    if !prefer_encrypt.is_empty() {
                         // If there are multiple headers, then just
                         // silently take the first one.
                         Some(prefer_encrypt[0].clone())
@@ -787,8 +783,7 @@ impl<'a> AutocryptSetupMessageParser<'a> {
                 ref p => return
                     Err(Error::MalformedMessage(
                         format!("Expected an MDC packet, got a {}",
-                                p.tag())
-                            .into())
+                                p.tag()))
                         .into()),
             }
 
@@ -802,8 +797,7 @@ impl<'a> AutocryptSetupMessageParser<'a> {
                 // has the right sequence of packets, but we haven't
                 // carefully checked the nesting.  We do that now.
                 if let Err(err) = pp.is_message() {
-                    return Err(err.context(
-                        "Invalid OpenPGP Message").into());
+                    return Err(err.context("Invalid OpenPGP Message"));
                 }
             }
             PacketParserResult::Some(pp) =>
@@ -814,7 +808,7 @@ impl<'a> AutocryptSetupMessageParser<'a> {
 
         // We're done!
         Ok(AutocryptSetupMessage {
-            prefer_encrypt: prefer_encrypt,
+            prefer_encrypt,
             passcode: self.passcode,
             passcode_format: self.passcode_format,
             passcode_begin: self.passcode_begin,
@@ -1109,4 +1103,27 @@ mod test {
                    &b"Testy McTestface <testy@example.org>"[..]);
         Ok(())
     }
+
+    /// Demonstrates a panic in the AutocryptHeader parser.
+    #[test]
+    fn issue_743() {
+        let data: Vec<u8> = vec![
+            0x41, 0x75, 0x02, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x20,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x00, 0x0a, 0xc4,
+            0x83, 0x40, 0x39, 0x0a, 0x38, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a, 0x0a,
+            0x20, 0x74, 0x6f, 0x63, 0x72, 0x79, 0x70, 0x74, 0x3a, 0x00, 0x00,
+            0x0a, 0x0a, 0x0a, 0x0a, 0x3d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x01, 0x35, 0x39, 0x33, 0x35, 0x38, 0x36, 0x34, 0x00, 0x00,
+            0x01, 0x00, 0x00, 0x00, 0x3e, 0x3f, 0x00, 0x08, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x11, 0x00, 0x3e, 0x08,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3e, 0x01, 0x08, 0x00,
+            0x00, 0x41, 0x75, 0x74, 0x6f, 0x63, 0x72, 0x79, 0x20, 0x02, 0x01,
+            0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x01, 0x70,
+        ];
+        let _ = AutocryptHeaders::from_bytes(&data);
+    }
 }
